# This function creates a project the same way like a React project.

import os
import subprocess
import kix

from zinc_cli.commands.aws_util.aws_utils import create_aws_service_model, bootstrap_cdk
from zinc_cli.commands.create.project.create_project_cmd import create_project
from zinc_cli.commands.create.project.create_project_request import CreateProjectRequest
from zinc_cli.commands.create.static_site.create_static_site_cmd import create_static_site
from zinc_cli.commands.create.static_site.create_static_site_request import CreateStaticSiteRequest
from zinc_cli.commands.create.zinc_create_request import ZincCreateRequest
from zinc_cli.infrastructure.models.infrastructure_service_model import InfrastructureServiceModel


def invoke():

    master_request: ZincCreateRequest = ZincCreateRequest()

    # Create local resources and template site.
    bucket_name = f"static.{master_request.domain}"
    project_request = CreateProjectRequest(
        project_name=master_request.project_name,
        domain_name=master_request.domain,
        bucket_name=bucket_name)
    create_project(project_request)

    # AWS Validation.
    service_model: InfrastructureServiceModel = create_aws_service_model()

    # Ensure that CDK is bootstrapped.
    bootstrap_cdk(service_model.aws_account_id.value, service_model.aws_region.value)

    # Add the static site request to the service model.
    static_site_request = CreateStaticSiteRequest(master_request.project_name, master_request.domain, bucket_name)
    service_model.append(create_static_site(static_site_request))

    # Create service infrastructure.
    create_infrastructure(service_model, master_request.dry_run)


def create_infrastructure(service_model: InfrastructureServiceModel, dry_run: bool):

    # Switch to infrastructure directory.
    current_path = _switch_to_infrastructure_path()

    # Create the infrastructure.
    _execute_infrastructure_command(dry_run, service_model)

    # Return to the previous path.
    os.chdir(current_path)
    kix.info(f"Execution Complete. Changed directory back to {current_path}.")


def _execute_infrastructure_command(dry_run, service_model):

    # Assemble the command.
    env_map = service_model.get_command_line_dict()
    deploy_command = "deploy" if dry_run is False else "synth"
    final_command = f"cdk {deploy_command} --require-approval never"
    kix.info(f"Executing command: {final_command}")

    # Prepare the environment variables for the sub process.
    env_map.update(os.environ.copy())

    # Run the command.
    result = subprocess.run(final_command.split(" "), env=env_map)
    kix.info(f"Executed result: {result}")


def _switch_to_infrastructure_path():
    module_path = os.path.dirname(__file__)
    current_path = os.getcwd()
    infrastructure_path = os.path.join(module_path, "..", "..", "infrastructure")
    os.chdir(infrastructure_path)
    kix.info(f"Changed directory to {infrastructure_path} to execute CDK.")
    return current_path



# -*- coding: utf-8 -*-

__author__ = "Jakrin Juangbhanich"
__email__ = "juangbhanich.k@gmail.com"
__version__ = "0.1.13"

from .zinc import main
from .commands import zinc_transform
from .commands.create import zinc_create



import kix

from .create_static_site_request import CreateStaticSiteRequest
from zinc_cli.commands.create.domain.domain_manager import DomainManager
from zinc_cli.commands.create.domain.hosted_zone_manager import HostedZoneManager
from zinc_cli.infrastructure.models.infrastructure_service_model import InfrastructureServiceModel


def create_static_site(request: CreateStaticSiteRequest):

    kix.info(f"Creating Static Site: {request.domain_name}")

    # Ensure the hosted zone.
    if not HostedZoneManager.has_hosted_zone(request.domain_name):
        kix.error(f"No hosted zone detected for {request.domain_name}."
                     f"Please ensure you have created a HostedZone for this domain on your AWS account.")
        raise Exception(f"No hosted zone on AWS account for {request.domain_name}")

    # Ensure the domain.
    if not DomainManager.validate(request.domain_name):
        raise Exception(f"Not a valid domain name: {request.domain_name}")

    if not DomainManager.user_owns_domain(request.domain_name):
        kix.warning(f"You do not own the domain {request.domain_name}. "
                    f"You will need to configure the domain's name servers to redirect the traffic.")

    # Return the instructions to CFN.
    service_model: InfrastructureServiceModel = InfrastructureServiceModel()
    service_model.create_static_site.set(True)
    service_model.domain_name.set(request.domain_name)
    service_model.project_name.set(request.project_name)
    service_model.static_site_bucket_name.set(request.bucket_name)
    return service_model



import os
import boto3
import kix
from zinc_cli.infrastructure.models.infrastructure_service_model import InfrastructureServiceModel


def create_aws_service_model():

    # Ensure the account credentials, then return the service model.
    service_model: InfrastructureServiceModel = InfrastructureServiceModel()

    try:
        account = boto3.client('sts').get_caller_identity().get('Account')
        my_region = "us-east-1"  # All accounts created in us-east-1.

        if account is None:
            raise Exception("AWS Account is not configured")

        if my_region is None:
            raise Exception("AWS Region is not configured")

        kix.info(f"Using AWS Account: {account}")
        kix.info(f"Using AWS Region: {my_region}")
        kix.info("If you would like to change accounts/regions, configure the default aws profile with 'aws configure'.")

        service_model.aws_account_id.set(account)
        service_model.aws_region.set(my_region)

        return service_model

    except Exception as e:
        kix.error("Error: ", str(e))
        raise Exception("Failed to get AWS credentials. Have you installed awscli and run 'aws configure'?")


def bootstrap_cdk(account_id: str, region: str):
    kix.info("Bootstrapping CDK")
    bootstrap = f"cdk bootstrap aws://{account_id}/{region}"
    kix.info(f"Bootstrap Command: {bootstrap}")
    os.system(bootstrap)



import kix
from aws_cdk import (
    core,
    aws_lambda,
    aws_apigateway,
    aws_certificatemanager, aws_route53, aws_route53_targets, aws_iam)
import os

from aws_cdk.aws_iam import PolicyStatement
from aws_cdk.custom_resources import AwsCustomResource, AwsSdkCall
from services.master_stack import CDKMasterStack


def add_contact_api(stack: CDKMasterStack, domain: str):

    module_path = os.path.dirname(__file__)
    lambda_path = os.path.join(module_path, "lambda")
    api_domain_name = f"api.{domain}"
    api_path = "contact"

    base_lambda = aws_lambda.Function(
        stack, 'ContactFormLambda',
        handler='lambda_handler.handler',
        runtime=aws_lambda.Runtime.PYTHON_3_7,
        environment={
            "TARGET_EMAIL": "juangbhanich.k@gmail.com",
            "SENDER_EMAIL": f"contact@{domain}",
            "SENDER_NAME": "Business Name"
        },
        code=aws_lambda.Code.asset(lambda_path),
    )

    base_lambda.add_to_role_policy(aws_iam.PolicyStatement(
        effect=aws_iam.Effect.ALLOW,
        resources=["*"],
        actions=["ses:SendEmail", "ses:SendRawEmail"]))

    certificate = create_api_certificate(stack, api_domain_name, stack.zone)

    verify_domain_create_call = AwsSdkCall(service="SES",
                                           action="verifyDomainIdentity",
                                           parameters={"Domain": domain},
                                           physical_resource_id_path="VerificationToken")

    verify_domain_identity = AwsCustomResource(
        stack, "VerifyDomainIdentity",
        on_create=verify_domain_create_call,
        policy_statements=[PolicyStatement(resources=["*"], actions=["ses:VerifyDomainIdentity"])])

    aws_route53.TxtRecord(
        stack, "SESVerificationRecord",
        zone=stack.zone,
        record_name=f"_amazonses.{domain}",
        values=[verify_domain_identity.get_data_string("VerificationToken")]
    )

    domain_options = aws_apigateway.DomainNameOptions(domain_name=api_domain_name, certificate=certificate)
    stage_options = aws_apigateway.StageOptions(
        throttling_rate_limit=10,
        throttling_burst_limit=100
    )

    base_api = aws_apigateway.RestApi(
        stack, 'ContactFormAPIGW',
        rest_api_name='ContactFormAPI',
        domain_name=domain_options,
        deploy_options=stage_options
    )

    kix.info("Routing A-Record Alias")
    a_record_target = aws_route53.RecordTarget.from_alias(aws_route53_targets.ApiGateway(base_api))
    aws_route53.ARecord(
        stack, "ApiAliasRecord",
        zone=stack.zone,
        target=a_record_target,
        record_name=api_domain_name)

    api_entity = base_api.root.add_resource(api_path)
    example_entity_lambda_integration = aws_apigateway.LambdaIntegration(
        base_lambda, proxy=False, integration_responses=[get_integration_response()])

    api_entity.add_method(
        'POST', example_entity_lambda_integration,
        method_responses=[get_method_response()])

    add_cors_options(api_entity)


def create_api_certificate(stack: core.Stack, domain: str, zone: aws_route53.HostedZone):
    kix.info("Creating Certificate")
    cert = aws_certificatemanager.DnsValidatedCertificate(
        stack, f"ApiCertificate",
        domain_name=domain,
        hosted_zone=zone)
    core.CfnOutput(stack, 'ApiCertificateArn', value=cert.certificate_arn)
    return cert


def add_cors_options(api_gateway_resource):
    api_gateway_resource.add_method(
        'OPTIONS',
        aws_apigateway.MockIntegration(
            integration_responses=[get_options_integration_response()],
            passthrough_behavior=aws_apigateway.PassthroughBehavior.WHEN_NO_MATCH,
            request_templates={"application/json": "{\"statusCode\":200}"}),
        method_responses=[get_options_method_response()])


def get_integration_response():
    integration_response = aws_apigateway.IntegrationResponse(
        status_code="200",
        response_parameters={'method.response.header.Access-Control-Allow-Origin': "'*'"})
    return integration_response


def get_method_response():
    method_response = aws_apigateway.MethodResponse(
        status_code="200",
        response_parameters={'method.response.header.Access-Control-Allow-Origin': True})
    return method_response


def get_options_integration_response():
    integration_response = aws_apigateway.IntegrationResponse(
        status_code="200",
        response_parameters={
            'method.response.header.Access-Control-Allow-Headers': "'Content-Type,X-Amz-Date,Authorization,X-Api-Key,X-Amz-Security-Token'",
            'method.response.header.Access-Control-Allow-Origin': "'*'",
            'method.response.header.Access-Control-Allow-Methods': "'GET,OPTIONS'"
        }
    )
    return integration_response


def get_options_method_response():
    method_response = aws_apigateway.MethodResponse(
        status_code="200",
        response_parameters={
            'method.response.header.Access-Control-Allow-Headers': True,
            'method.response.header.Access-Control-Allow-Methods': True,
            'method.response.header.Access-Control-Allow-Origin': True,
        }
    )
    return method_response



import os
from shutil import rmtree

OUTPUT_DIR = "output"


def redirect_output() -> str:
    if os.path.exists(OUTPUT_DIR):
        rmtree(OUTPUT_DIR)

    os.mkdir(OUTPUT_DIR)
    os.chdir(OUTPUT_DIR)
    return os.getcwd()


def clear_output(original_dir: str):
    os.chdir(original_dir)
    if os.path.exists(OUTPUT_DIR):
        rmtree(OUTPUT_DIR)



import boto3
import kix


class HostedZoneManager:

    @staticmethod
    def has_hosted_zone(domain: str):
        client = boto3.client('route53')
        zones = client.list_hosted_zones_by_name(DNSName=domain)

        kix.info(f"Got hosted zones: {zones}")
        if not zones:
            return False

        hosted_zones = zones['HostedZones']
        kix.info(f"Got hosted zones: {hosted_zones}")
        if len(hosted_zones) == 0:
            return False

        # Check if zone exists in the response.
        # AWS returns the requested zone as index 0 if it exists.
        if domain in hosted_zones[0]["Name"]:
            return True

        return False



import json
import shutil
import subprocess
import os
import kix

from.create_project_request import CreateProjectRequest


def create_project(request: CreateProjectRequest):
    # Download the template into the directory.

    CLONE_REPO = "https://github.com/krinj/zinc-react-template.git"
    FOLDER_NAME = "zinc-react-template"
    DESTINATION = "site-template"

    _create_local_resources(request.project_name)
    _clone_template_site(CLONE_REPO, DESTINATION, FOLDER_NAME)
    _inject_deployment_script(DESTINATION, request)


def _inject_deployment_script(destination: str, request: CreateProjectRequest):
    # Replace the deployment script.
    package_json_path = os.path.join(destination, "package.json")
    with open(package_json_path, "r") as f:
        current_package_json = json.load(f)

    # Inject the deployment script.
    deploy_script = f"aws s3 sync build/ s3://{request.bucket_name} --acl public-read"
    current_package_json["scripts"]["deploy"] = deploy_script
    with open(package_json_path, "w") as f:
        json.dump(current_package_json, f, indent=2)

    # Inject the API endpoint.
    api_endpoint = f"https://api.{request.domain_name}"

    kix.info("package.json updated with deployment script")


def _clone_template_site(repo_address: str, destination: str, source_path: str):
    kix.info(f"Creating project in {os.getcwd()}")

    # Clone the repo.
    kix.info(f"Cloning template site from: {repo_address}")
    subprocess.call(["git", "clone", repo_address])

    # Copy the template folder.
    app_folder_path = os.path.join(source_path, "app")
    shutil.copytree(app_folder_path, destination)
    kix.info(f"Copied template source from {app_folder_path} to {destination}")

    # Delete source.
    shutil.rmtree(source_path)
    kix.info(f"Source removed: {source_path}")


def _create_local_resources(project_name: str):
    path = project_name

    # Check if the directory already exists.
    if os.path.exists(path):
        message = f"Cannot create project {project_name}. The directory {project_name} already exists."
        kix.error(message)
        raise IsADirectoryError(message)

    # It doesn't exist, so we can try to make the project here.
    os.mkdir(path)
    os.chdir(path)
    kix.info(f"Project path changed to: {os.getcwd()}")



import argparse
import os
from typing import TextIO

TEMPLATE_TOKEN = "<INSERT_TOKEN>"
INDENT_CHAR = "    "


def content_template():
    x = """import ZincContentInterface from "./zincContentInterface";
    
    class ZincContent extends ZincContentInterface {
        constructor () {
            super();
            <INSERT_TOKEN>
        }
    }
    
    export default ZincContent
    """
    return x


def invoke():
    print("Creating Project...")

    parser = argparse.ArgumentParser()
    parser.add_argument("-t", "--template_path", type=str, required=True, help="Path to the front-end template to transform into.")
    args = parser.parse_args()

    template_path = args.template_path
    print("tp: " + template_path)
    transform_project(template_path)


def transform_project(template_path: str = ""):
    # Transforms a project into a TypeScript object so a front-end can import it.
    path = os.path.join(template_path, "zinc")
    if not os.path.exists(path):
        os.makedirs(path, exist_ok=True)

    path = os.path.join(path, "zincContent.ts")

    template = content_template()

    with open(path, "w") as file:

        indent_level = 0

        for template_line in template.split("\n"):
            template_line = template_line.strip()

            if TEMPLATE_TOKEN in template_line:
                write_content(file, indent_level)
            else:
                indent_level = process_template_line(file, indent_level, template_line)


def process_template_line(file: TextIO, indent_level: int, template_line: str):

    if "}" in template_line:
        indent_level -= 1

    file.write(get_indent(indent_level))
    file.write(template_line + "\n")

    if "{" in template_line:
        indent_level += 1

    return indent_level


def get_indent(level: int):
    return level * INDENT_CHAR


def write_content(file: TextIO, indent_level: int):
    for i in range(3):
        file.write(get_indent(indent_level))
        file.write("this.addBody('This is injected content!');")
        file.write("\n")



import json
import boto3
from botocore.exceptions import ClientError
import os


TARGET_EMAIL_KEY = "TARGET_EMAIL"


def handler(event, context):

    notes = event["notes"] if "notes" in event else "No Notes"
    phone = event["phone"] if "phone" in event else "No Phone"
    name = event["name"] if "name" in event else "No Name"
    email = event["email"] if "email" in event else "hello@zinccli.com"
    email = "hello@zinccli.com" if (len(email) == 0 or "@" not in email) else email

    # Replace sender@example.com with your "From" address.
    # This address must be verified with Amazon SES.
    SENDER = "Zinc Admin <hello@zinccli.com>"

    # Replace recipient@example.com with a "To" address. If your account
    # is still in the sandbox, this address must be verified.
    if TARGET_EMAIL_KEY not in os.environ:
        return {"statusCode": "500", "body": "No target email detected."}

    RECIPIENT = os.environ[TARGET_EMAIL_KEY]

    # If necessary, replace us-west-2 with the AWS Region you're using for Amazon SES.
    AWS_REGION = "us-east-1"

    # The subject line for the email.
    SUBJECT = f"{name} @ Amazon SES Test (SDK for Python)"

    # The email body for recipients with non-HTML email clients.
    BODY_TEXT = ("Amazon SES Test (Python)\r\n"
                 "This email was sent with Amazon SES using the "
                 "AWS SDK for Python (Boto)." + notes
                 )

    # The HTML body of the email.
    BODY_HTML = f"""<html>
    <head></head>
    <body>
      <h1>Amazon SES Test (SDK for Python)</h1>
      <p>{name}</p>
      <p>{phone}</p>
      <p>{email}</p>
      <p>{notes}</p>
      <p>This email was sent with
        <a href='https://aws.amazon.com/ses/'>Amazon SES</a> using the
        <a href='https://aws.amazon.com/sdk-for-python/'>
          AWS SDK for Python (Boto)</a>.</p>
    </body>
    </html>
                """

    # The character encoding for the email.
    CHARSET = "UTF-8"

    # Create a new SES resource and specify a region.
    client = boto3.client('ses', region_name=AWS_REGION)

    response = {
        'statusCode': 200,
        "body": "No event body was found.",
        "event": event
    }

    # Try to send the email.
    try:
        # Provide the contents of the email.
        ses_response = client.send_email(
            Destination={
                'ToAddresses': [
                    RECIPIENT,
                ],
            },
            Message={
                'Body': {
                    'Html': {
                        'Charset': CHARSET,
                        'Data': BODY_HTML,
                    },
                    'Text': {
                        'Charset': CHARSET,
                        'Data': BODY_TEXT,
                    },
                },
                'Subject': {
                    'Charset': CHARSET,
                    'Data': SUBJECT,
                },
            },
            ReplyToAddresses=[
                email,
            ],
            Source=SENDER
        )

    # Display an error if something goes wrong.
    except ClientError as e:
        print(e.response['Error']['Message'])
        response["ses_response"] = e.response['Error']['Message']
    else:
        print("Email sent! Message ID:"),
        print(ses_response['MessageId'])
        response["ses_response"] = "Successful mail sent!"

    return response



from aws_cdk import core, aws_lambda
import os


class CDKBookingsStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, project_id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)
        module_path = os.path.dirname(__file__)

        base_lambda = aws_lambda.Function(
            self, f"GetFreeSlots",
            handler="lambda_handler.handler",
            runtime=aws_lambda.Runtime.PYTHON_3_7,
            code=aws_lambda.Code.asset(f"{module_path}/lambda"),
        )



from typing import Dict, Union

import kix

from .infrastructure_service_field import InfrastructureServiceField as Field


class InfrastructureServiceModel:
    def __init__(self):
        self._all_fields: Dict[str, Field] = {}

        # General.
        self.aws_account_id: Field = self._add_field("Z_AWS_ACCOUNT_ID")
        self.aws_region: Field = self._add_field("Z_AWS_REGION")
        self.project_name: Field = self._add_field("Z_PROJECT_NAME")

        # Static Site Creation.
        self.create_static_site: Field = self._add_field("Z_CREATE_STATIC_SITE", default=False)
        self.domain_name: Field = self._add_field("Z_DOMAIN_NAME")
        self.static_site_bucket_name: Field = self._add_field("Z_STATIC_SITE_BUCKET")

        # CRUD API Creation.
        self.create_crud_api: Field = self._add_field("Z_CREATE_CRUD_API", default=False)

        # Contact Form API Creation.
        self.create_contact_api: Field = self._add_field("Z_CREATE_CONTACT_API", default=False)

    def _add_field(self, key: str, default: Union[str, bool] = "") -> Field:
        field = Field(key, default)
        self._all_fields[key] = field
        return field

    def save_to_environ(self):
        for field in self._all_fields.values():
            field.write_to_environ()

    def load_from_environ(self):
        data = {}
        for field in self._all_fields.values():
            field.load_from_environ()
            data[field.key] = field.value

        kix.info("Loading Infrastructure Model from Environment", data)

    def append(self, new_model: 'InfrastructureServiceModel'):

        # Absorb the edited fields from the new model.
        for k, field in self._all_fields.items():
            new_field = new_model._all_fields[k]
            if new_field.was_edited:
                field.set(new_field.value)

    def get_command_line_args(self) -> str:
        # Turn all the non-default fields into environment arguments.
        commands: Dict[str, str] = self.get_command_line_dict()
        commands_list = [f"{k}={v}" for k, v in commands.items()]
        return " ".join(commands_list)

    def get_command_line_dict(self) -> Dict[str, str]:
        # Turn all the non-default fields into environment arguments.
        commands: Dict[str, str] = {}
        for field in self._all_fields.values():
            if field.was_edited:
                commands[field.key] = str(field.value)
        return commands



import os

import kix
from aws_cdk import core
from aws_cdk.aws_route53 import HostedZone


class CDKMasterStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, domain: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)
        self.stack_module_path: str = os.path.dirname(__file__)

        self.zone: HostedZone = HostedZone.from_lookup(
            self, "HostedZone",
            domain_name=domain,
            private_zone=False)

        kix.info(f"Zone from look-up: {self.zone.zone_name}")



import os
from typing import Union


class InfrastructureServiceField:
    def __init__(self, key: str, default_value: Union[str, bool] = None):
        self.key: str = key
        self.value: Union[str, bool] = default_value
        self.was_edited: bool = False

    def set(self, value: Union[str, bool]):
        self.value = value
        self.was_edited = True
        if type(self.value) is not str and type(self.value) is not bool:
            raise Exception(f"The value for {self.key} must be str or bool, but is instead {type(self.value)}.")

    def write_to_environ(self):
        os.environ[self.key] = str(self.value)

    def load_from_environ(self):
        if self.key in os.environ:

            # Load from the environment.
            self.value = os.environ[self.key]

            # Able to serialize to and from a bool.
            if self.value == str(True):
                self.value = True

            if self.value == str(False):
                self.value = False



import kix

from .create_contact_api_request import CreateContactApiRequest
from zinc_cli.infrastructure.models.infrastructure_service_model import InfrastructureServiceModel


def create_contact_api(request: CreateContactApiRequest):

    kix.info("Creating Contact API")

    # Return the instructions to CFN.
    service_model: InfrastructureServiceModel = InfrastructureServiceModel()
    service_model.project_name.set(request.project_name)
    service_model.create_contact_api.set(True)
    return service_model



class CreateProjectRequest:
    def __init__(self, project_name: str, domain_name: str, bucket_name: str):
        self.project_name: str = project_name
        self.bucket_name: str = bucket_name
        self.domain_name: str = domain_name



import kix

from .create_crud_api_request import CreateCrudApiRequest
from zinc_cli.infrastructure.models.infrastructure_service_model import InfrastructureServiceModel


def create_crud_api(request: CreateCrudApiRequest):

    kix.info("Creating CRUD API")

    # Return the instructions to CFN.
    service_model: InfrastructureServiceModel = InfrastructureServiceModel()
    service_model.project_name.set(request.project_name)
    service_model.create_crud_api.set(True)
    return service_model



import kix
from aws_cdk import core, aws_route53, aws_s3, aws_certificatemanager, aws_cloudfront, aws_route53_targets, \
    aws_s3_deployment
import os

from services.master_stack import CDKMasterStack


def add_static_site(stack: CDKMasterStack, domain: str, bucket_name: str):
    module_path = os.path.dirname(__file__)

    # Construct code goes here
    core.CfnOutput(stack, "Site", value=f"https://{domain}")

    # Content bucket
    kix.info("Bucket Name: " + bucket_name)
    site_bucket = aws_s3.Bucket(
        stack, "SiteBucket",
        bucket_name=bucket_name,
        website_index_document="index.html",
        website_error_document="index.html",
        public_read_access=True,
        removal_policy=core.RemovalPolicy.DESTROY)
    core.CfnOutput(stack, "BucketArn", value=site_bucket.bucket_arn)

    # Certificate
    kix.info("Creating Certificate")
    cert = aws_certificatemanager.DnsValidatedCertificate(
        stack, f"{id}-bucket",
        domain_name=domain,
        hosted_zone=stack.zone)
    core.CfnOutput(stack, 'CertificateArn', value=cert.certificate_arn)

    kix.info("Creating Distribution")
    distribution = aws_cloudfront.CloudFrontWebDistribution(
        stack, "SiteDistribution",
        alias_configuration=aws_cloudfront.AliasConfiguration(
            acm_cert_ref=cert.certificate_arn,
            names=[domain],
            ssl_method=aws_cloudfront.SSLMethod.SNI,
            security_policy=aws_cloudfront.SecurityPolicyProtocol.TLS_V1_1_2016,
        ),
        origin_configs=[
            aws_cloudfront.SourceConfiguration(
                s3_origin_source=aws_cloudfront.S3OriginConfig(s3_bucket_source=site_bucket),
                behaviors=[aws_cloudfront.Behavior(is_default_behavior=True)]
            )])
    core.CfnOutput(stack, "DistributionId", value=distribution.distribution_id)
    a_record_target = aws_route53.AddressRecordTarget.from_alias(aws_route53_targets.CloudFrontTarget(distribution))

    # Route 53 alias record for the CloudFront distribution
    kix.info("Routing A-Record Alias")
    aws_route53.ARecord(
        stack, "SiteAliasRecord",
        zone=stack.zone,
        target=a_record_target,
        record_name=domain)

    kix.info("Sample Bucket Deployment")
    deploy_name = "DeployWithInvalidation"
    deploy_source = [aws_s3_deployment.Source.asset(os.path.join(module_path, "default_source/"))]

    aws_s3_deployment.BucketDeployment(
        stack, deploy_name,
        sources=deploy_source,
        destination_bucket=site_bucket,
        distribution=distribution,
        distribution_paths=["/*"])




import kix
from aws_cdk import (
    core,
    aws_lambda,
    aws_apigateway,
    aws_certificatemanager, aws_route53, aws_route53_targets)
import os


class CDKCrudApiStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, domain: str, **kwargs, ) -> None:
        super().__init__(scope, id, **kwargs)
        module_path = os.path.dirname(__file__)
        lambda_path = os.path.join(module_path, "lambda")
        api_domain_name = f"api.{domain}"

        base_lambda = aws_lambda.Function(
            self, 'ApiCorsLambda',
            handler='lambda_handler.handler',
            runtime=aws_lambda.Runtime.PYTHON_3_7,
            code=aws_lambda.Code.asset(lambda_path),
        )

        zone = aws_route53.HostedZone.from_lookup(
            self, "HostedZone",
            domain_name=domain,
            private_zone=False)
        kix.info(f"Zone from look-up: {zone.zone_name}")

        certificate = self.create_certificate(api_domain_name, zone)

        api_domain_name = f"api.{domain}"
        domain_options = aws_apigateway.DomainNameOptions(domain_name=api_domain_name, certificate=certificate)
        stage_options = aws_apigateway.StageOptions(
            throttling_rate_limit=10,
            throttling_burst_limit=100
        )
        base_api = aws_apigateway.RestApi(
            self, 'ApiGatewayWithCors',
            rest_api_name='ApiGatewayWithCors',
            domain_name=domain_options,
            deploy_options=stage_options
        )

        kix.info("Routing A-Record Alias")
        a_record_target = aws_route53.RecordTarget.from_alias(aws_route53_targets.ApiGateway(base_api))
        aws_route53.ARecord(
            self, "ApiAliasRecord",
            zone=zone,
            target=a_record_target,
            record_name=api_domain_name)

        example_entity = base_api.root.add_resource('example')
        example_entity_lambda_integration = aws_apigateway.LambdaIntegration(
            base_lambda, proxy=False, integration_responses=[self.get_integration_response()])

        example_entity.add_method(
            'GET', example_entity_lambda_integration,
            method_responses=[self.get_method_response()])

        self.add_cors_options(example_entity)

    def create_certificate(self, domain: str, zone: aws_route53.HostedZone):
        kix.info("Creating Certificate")
        cert = aws_certificatemanager.DnsValidatedCertificate(
            self, f"ApiCertificate",
            domain_name=domain,
            hosted_zone=zone)
        core.CfnOutput(self, 'CertificateArn', value=cert.certificate_arn)
        return cert

    @staticmethod
    def add_cors_options(api_gateway_resource):
        api_gateway_resource.add_method(
            'OPTIONS',
            aws_apigateway.MockIntegration(
                integration_responses=[CDKCrudApiStack.get_options_integration_response()],
                passthrough_behavior=aws_apigateway.PassthroughBehavior.WHEN_NO_MATCH,
                request_templates={"application/json": "{\"statusCode\":200}"}),
            method_responses=[CDKCrudApiStack.get_options_method_response()])

    @staticmethod
    def get_integration_response():
        integration_response = aws_apigateway.IntegrationResponse(
            status_code="200",
            response_parameters={'method.response.header.Access-Control-Allow-Origin': "'*'"})
        return integration_response

    @staticmethod
    def get_method_response():
        method_response = aws_apigateway.MethodResponse(
            status_code="200",
            response_parameters={'method.response.header.Access-Control-Allow-Origin': True})
        return method_response

    @staticmethod
    def get_options_integration_response():
        integration_response = aws_apigateway.IntegrationResponse(
            status_code="200",
            response_parameters={
                'method.response.header.Access-Control-Allow-Headers': "'Content-Type,X-Amz-Date,Authorization,X-Api-Key,X-Amz-Security-Token'",
                'method.response.header.Access-Control-Allow-Origin': "'*'",
                'method.response.header.Access-Control-Allow-Methods': "'GET,OPTIONS'"
            }
        )
        return integration_response

    @staticmethod
    def get_options_method_response():
        method_response = aws_apigateway.MethodResponse(
            status_code="200",
            response_parameters={
                'method.response.header.Access-Control-Allow-Headers': True,
                'method.response.header.Access-Control-Allow-Methods': True,
                'method.response.header.Access-Control-Allow-Origin': True,
            }
        )
        return method_response



import subprocess
import boto3
from botocore.client import BaseClient

# ======================================================================================================================
# Singleton Methods.
# ======================================================================================================================


class DomainManager:

    _AWS_CLIENT = None

    @staticmethod
    def _client() -> BaseClient:
        if DomainManager._AWS_CLIENT is None:
            DomainManager._AWS_CLIENT = boto3.client("route53domains", region_name="us-east-1")
        return DomainManager._AWS_CLIENT

    @staticmethod
    def validate(domain_name: str):
        try:
            domain_name = domain_name.lower()
            domain_split = domain_name.split(".")
            domain_sections = len(domain_split)
            if domain_sections < 2 or domain_sections > 3:
                return False

            for domain_str in domain_split:
                if not domain_str.isalnum():
                    return False

            return True
        except Exception as e:
            print("validation failure", e)
        return False

    @staticmethod
    def user_owns_domain(domain_name: str):
        is_domain_owned = False

        try:
            response = DomainManager._client().get_domain_detail(DomainName=domain_name)
            is_domain_owned = True
            print(response)
        except Exception as e:
            print("Unable to find domain: ", str(e))

        print(f"Domain Check Result: {domain_name}, owned: {is_domain_owned}")
        return is_domain_owned

    @staticmethod
    def is_domain_available(domain_name: str):

        is_available = False

        try:
            response = DomainManager._client().check_domain_availability(DomainName=domain_name)
            availability_status = response["Availability"]
            is_available = availability_status == "AVAILABLE"
            print(availability_status)
        except Exception as e:
            print("Unable to find domain: ", str(e))

        return is_available



import argparse
import kix


class ZincCreateRequest:

    def __init__(self):

        # Core project arguments.
        self.project_name: str = "untitled"
        self.domain: str = "blah.com"

        # Service APIs.
        self.with_contact_api: bool = True

        # Meta-data flags.
        self.dry_run: bool = True
        self.wizard: bool = False

    def gather_arguments(self):
        self._capture_cli_arguments()
        self._execute_wizard()

    def _capture_cli_arguments(self):
        parser = argparse.ArgumentParser()
        parser.add_argument("-n", "--name", type=str, required=True, help="Name of the new project.")
        parser.add_argument("--domain", type=str, required=True, help="Bootstrap a static site at the domain.")
        parser.add_argument("--with-contact-api", action="store_true",
                            help="Whether or not to attach the contact form API.")
        parser.add_argument("--dry-run", action="store_true", help="Do not publish to actual AWS.")
        parser.add_argument("--wizard", action="store_true", help="On-board with the wizard.")
        args = parser.parse_args()

        self.project_name: str = args.name
        self.domain: str = args.domain
        self.with_contact_api: str = args.with_contact_api

        self.dry_run: bool = args.dry_run
        self.wizard: bool = args.wizard

        kix.info(f"Project Name: {self.project_name}")
        kix.info(f"Domain Name: {self.domain}")
        kix.info(f"Dry-run: {str(self.dry_run)}")

    def _execute_wizard(self):
        # Step by step input for each of the arguments.
        if self.wizard:
            return

        kix.warning("Wizard is not implemented yet.")



#!/usr/bin/env python3
from aws_cdk import core
from models.infrastructure_service_model import InfrastructureServiceModel
from services.contact_api.cdk_contact_api_stack import add_contact_api
from services.master_stack import CDKMasterStack
from services.static_site.cdk_static_site_stack import add_static_site


def build(m: InfrastructureServiceModel):

    # This is the build definition.
    app = core.App()

    # Prepare the environment variables.
    env = {"account": m.aws_account_id.value, "region": m.aws_region.value}
    stack_id = f"{m.project_name.value}-zinc-stack"

    # Create the master stack.
    master_stack = CDKMasterStack(app, stack_id, domain=m.domain_name.value, env=env)

    # Static site.
    if m.create_static_site.value:
        add_static_site(master_stack, domain=m.domain_name.value, bucket_name=m.static_site_bucket_name.value)

    # Contact Form API.
    if m.create_contact_api.value:
        add_contact_api(master_stack, domain=m.domain_name.value)

    # Synthesize the application.
    app.synth()


def main():
    print("Deploying CDK Stack")
    # Load the service model from the environment.
    service_model: InfrastructureServiceModel = InfrastructureServiceModel()
    service_model.load_from_environ()
    build(service_model)


if __name__ == "__main__":
    main()
